# Fansi.py
"""
A library for easily adding colour, emphasis
and emojis to your terminal printouts.

By Daniel Rivas, 2018.
"""

import re
from colorama import init, deinit
from .data import AnsiCodes, EmojiCodes


def parse_emphasis(str, resetcode=AnsiCodes.reset()):
    """
    Italics with _tags_ or *tags*.
    Bold with __tags__ or **tags**.
    Bold and italic with ___tags___ or ***tags***
    """

    strong_emphasis = re.compile(
        r"(?<!\w)(_{3}|\*{3})(?!_)(.+?)(?<!_)\1(?!\w)")
    str = strong_emphasis.sub(
        AnsiCodes.form_code("bold italics") + r"\2" + AnsiCodes.reset() +
        resetcode, str)

    strong = re.compile(r"(?<!\w)(_{2}|\*{2})(?!_)(.+?)(?<!_)\1(?!\w)")
    str = strong.sub(
        AnsiCodes.form_code("bold") + r"\2" + AnsiCodes.reset() + resetcode,
        str)

    emphasis = re.compile(r"(?<!\w)(_{1}|\*{1})(?!_)(.+?)(?<!_)\1(?!\w)")

    str = emphasis.sub(
        AnsiCodes.form_code("italics") + r"\2" + AnsiCodes.reset() + resetcode,
        str)

    return str


def parse_inline_styles(str, resetcode=AnsiCodes.reset()):
    """
    ::red bg-blue bold::This text is affected::reset::This code is not.
    """

    tag_re = re.compile(r" (?<!\w)(:{2})(?!:)(.+?)(?<!:)\1(?!\w)")
    str = tag_re.sub(lambda match: check_for_end(match.group(2), resetcode),
                     str)

    return str


def check_for_end(str, resetcode):
    if str == "end":
        return AnsiCodes.reset() + resetcode
    else:
        return AnsiCodes.form_code(str)


def parse_emojis(str):
    """
    TODO: Parse emojis in the text.
    TODO: Syntax :emoji:
    """
    emoji_re = re.compile(r"(?<!\w)(:{1})(?!:)(.+?)(?<!:)\1(?!\w)")
    str = emoji_re.sub(lambda match: EmojiCodes.get_emoji(match.group(2)), str)
    return str


def format(str, codes=None):
    if codes is not None:
        reset = AnsiCodes.form_code(codes)
        str = parse_emphasis(str, reset)
        str = parse_inline_styles(str, reset)
        str = parse_emojis(str)
        str = AnsiCodes.form_code(codes) + str + AnsiCodes.reset()

    str = parse_emphasis(str)
    str = parse_inline_styles(str)
    str = parse_emojis(str)
    str = AnsiCodes.reset() + str + AnsiCodes.reset()

    return str


def say(str, codes=None):
    init()
    str = format(str, codes)
    print(str)
    deinit()


# --- Premade formats ---


def danger(str, emoji=True):
    """
    For urgent messages, e.g. error printouts.
    """
    say(str, "bold red")



# -*- coding: utf-8 -*-
from distutils.core import setup

packages = \
['fansi']

package_data = \
{'': ['*']}

install_requires = \
['colorama>=0.4.0,<0.5.0']

setup_kwargs = {
    'name': 'fansi',
    'version': '0.1.4',
    'description': 'A Python library that makes formatting, colouring and adding emojis to terminal printouts easy.',
    'long_description': '# Fansi\nFansi is a Python library that makes formatting, colouring and adding emojis to terminal printouts easy.\n\n![fansi demo](https://i.postimg.cc/zGcL6MhG/Screenshot-2018-10-26-at-13-40-19.png)\n\n# Installation\n\n`pip install fansi`\n\nTo uninstall: `pip uninstall fansi`\n\n# Usage\n## First steps\nFirst, include fansi in your script: \n\n> `from fansi import fansi`\n\nThen, instead of `print()` statements, use `fansi.say()`.\n\n> `fansi.say("This string can have fansi formatting!")`\n\nIf you want to generate a formatted string without printing it, use `fansi.format()`.\n\n> `a = fansi.format("Here\'s a string ::blue:: formatted with Ansi characters! ::end:: It\'s partly blue!")`\n\n## Basic formatting\n\nTo add italics, use `_tags_` or `*tags*`. To add bolding, use `__tags__` or `**tags**`. To add bolding and italics, use `___tags___` or `***tags***`.\n\nFor example:\n> `fansi.say("This _text will be italicised_. This text will be **bolded**. ___This text will be both___.")`\n\n## Inline tagging\n\nYou can add Ansi formatting characters inline, affecting the colour or emphasis of your text, using `::tags::` To reset the formatting, use the `::end::` tag.\n\nFor example: \n> `fansi.say("This text will be normal. ::green bold italics:: This text will be green, bold and italicised. ::end:: This text will be back to normal.")`\n\n## Global tags\n\nFansi tags can also be added to the entire string.\n\nFor example: `fansi.say("This text will be magenta on a blue background.", "magenta bg-blue")`\n\n## Emojis\n\nYou can also add any emoji, using `:tags:`. Long and shortnames work.\n\nFor example: \n> `fansi.say("Here are some :poop: emojis! :panda_face::tiger::cat:")`\'\n\nThat\'s it!\n\n# Appendix: Fansi tags\n\n| Tag          | Description           |\n|--------------|-----------------------|\n| `bold`       | Bold text             |\n| `italics`    | Italicised text       |\n| `underline`  | Underlined text       |\n| `blink`      | Blinking text (don\'t) |\n| `invisible`  | Invisible text        |\n|--------------|-----------------------|\n| `black`      | Black text            |\n| `red`        | Red text              |\n| `green`      | Green text            |\n| `yellow`     | Yellow text           |\n| `blue`       | Blue text             |\n| `magenta`    | Magenta text          |\n| `cyan`       | Cyan text             |\n| `white`      | White text            |\n|--------------|-----------------------|\n| `bg-black`   | Black background      |\n| `bg-red`     | Red background        |\n| `bg-green`   | Green background      |\n| `bg-yellow`  | Yellow background     |\n| `bg-blue`    | Blue background       |\n| `bg-magenta` | Magenta background    |\n| `bg-cyan`    | Cyan background       |\n| `bg-white`   | White background      |\n',
    'author': 'Daniel Rivas Perez',
    'author_email': 'drivas12@googlemail.com',
    'url': 'https://github.com/drivasperez/fansi/',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.0,<4.0',
}


setup(**setup_kwargs)
